import json
import re
from .ccl_tt_tree import CclTtTree
from .data_base import MyDatabase
from .ccl_tt_node import CclTtNode, CclTtNodes
from collections import OrderedDict


class CclManager:
    def __init__(self):
        self.db = MyDatabase()
        self.trees = {}
        self.init_tree("source")
    
    def init_tree(self, name: str):
        if name in self.trees:
            return False
        tree = CclTtTree(name, self.db)
        self.trees[name] = tree
        return tree

def build_tree(father:CclTtNodes,data):
    name = data["name"]
    ty = data.get("type", None)
    value = data.get("value", None)
    if ty is not None or value is not None:
        if ty == "int":
            father.add(name, int(value))
        elif ty == "bool":
            father.add(name, bool(value))
        elif ty == "str":
            father.add(name, str(value))
        elif ty == "real":
            father.add(name, float(value))
        elif ty == "int list":
            father.add(name, [int(val) for val in value])
        elif ty == "real list":
            father.add(name, [float(val) for val in value])
    else:
        father.add(name)
    exprn = data.get("exprn", None)
    if exprn is not None:
        child = father.get(name)
        child.add_exprn(exprn)
    node = father.get(name)
    for child_data in data.get('children', []):
        build_tree(node,child_data)


def parse_config_to_dict(config_text):
    #先创建一个根节点
    root = {
        "name": "root",
        "children": []
    }
    current_node = root
    stack = [current_node]
    lines = config_text.splitlines()
    for line in lines:
        line = line.strip()
        if not line:
            continue
        if line.startswith("//"):
            continue
        #如果这一行是一个名字，并且接下来一行是一个{
        if re.match(r'^[a-zA-Z_][a-zA-Z0-9_]*\s*:?\s*[a-zA-Z0-9_]*$', line):
            new_node = {
                "name": line,
                "children": []
            }
            current_node["children"].append(new_node)
            stack.append(current_node)
        elif line == "{":
            current_node = new_node
        elif line == "}":
            if stack:
                current_node = stack[-1]
                stack.pop()
            else:
                current_node = root
        elif re.match(r'^[a-zA-Z_][a-zA-Z0-9_]*\s*=\s*.*$', line):
            #如果这一行是一个名字=值
            name, value = line.split("=", 1)
            name = name.strip()
            value = value.strip()
            name = name.strip('"')
            value = value.strip('"')
            type = "str"
            if value.lower() == "true":
                value = True
                type = "bool"
            elif value.lower() == "false":
                value = False
                type = "bool"
            elif re.match(r'^\d+$', value):
                value = int(value)
                type = "int"
            elif re.match(r'^\d+\.\d+$', value):
                value = float(value)
                type = "real"
            #1.0e-5
            elif re.match(r'^\d+\.?\d*[eE][+-]?\d+$', value):
                value = float(value)
                type = "real"
            # 先检查是否为单个浮点数（包括负数）
            elif re.match(r'^[+-]?(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?$', value):
                value = float(value)
                type = "real"
            #0.0,0.0,1.0
            elif re.match(r'^\d+(,\d+)*$', value):
                value = [int(val) for val in value.split(",")]
                type = "int list"
            # 再检查是否为浮点数列表（确保有逗号分隔符）
            elif re.match(r'^[+-]?\d+(?:\.\d+)?(?:[eE][+-]?\d+)?(?:\s*,\s*[+-]?\d+(?:\.\d+)?(?:[eE][+-]?\d+)?)+$', value):
                value = [float(v) for v in value.split(",")]
                type = "real list"
            #velocity[0]=0.0;velocity[1]=0.0;velocity[2]=0.0;
            elif re.match(r'(\w+\[?\d*\]?)=("[^"]*"|[^;]+)', value):
                pattern = r'(\w+\[?\d*\]?)=("[^"]*"|[^;]+)'
                value_node = {
                    "name": name,
                    "children": []
                }
                for key, v in re.findall(pattern, value):
                    key = key.strip()
                    v = v.strip()
                    type = "str"
                    if v.startswith('"') and v.endswith('"'):
                        v = v[1:-1]
                        type = "str"
                    if re.match(r'^\d+$', v):
                        v = int(v)
                        type = "int"
                    elif re.match(r'^\d+\.\d+$', v):
                        v = float(v)
                        type = "real"
                    elif re.match(r'^\d+\.?\d*[eE][+-]?\d+$', v):
                        v = float(v)
                        type = "real"
                    elif re.match(r'^\d+(,\d+)*$', v):
                        v = [int(val) for val in v.split(",")]
                        type = "int list"
                    elif re.match(r'^\d+(\.\d+)?(,\d+(\.\d+)?)*$', v):
                        v = [float(val) for val in v.split(",")]
                        type = "real list"
                    value_node["children"].append({"name": key, "value": v, "type": type})
                current_node["children"].append(value_node)
                continue

            value_node = {
                "name": name,
                "value": value,
                "type": type,
            }
            current_node["children"].append(value_node)
        else:
            print(f"无法解析的行: {line}")
            return None
    return root
        


def build_tree_from_json(path: str):
    with open(path, "r", encoding="utf-8") as f:
        config_text = f.read()
    root =  parse_config_to_dict(config_text)
    json_str = json.dumps(root, indent=4, ensure_ascii=False)
    print(json_str)
    data = json.loads(json_str, object_pairs_hook=OrderedDict)
    manager = CclManager()
    tree = manager.trees["source"]
    root = tree.ROOT()
    for child_data in data.get('children', []):
        build_tree(root,child_data)
    return manager

def merge_lines_with_slash(lines):
    result = []
    buffer = ''
    skip = False

    for i, line in enumerate(lines):
        if skip:
            skip = False
            continue

        line = line.rstrip('\n')

        if line.endswith('\\'):
            if i + 1 < len(lines):
                merged = line[:-1].rstrip() + ' ' + lines[i + 1].lstrip()
                result.append(merged)
                skip = True
            else:
                result.append(line[:-1])  # 如果最后一行以 / 结尾
        else:
            result.append(line)

    return result

def parse_ccl_to_dict(ccl_text):
    lines = merge_lines_with_slash(ccl_text.splitlines())
    root = {
        "name": "root",
        "children": []
    }
    current_node = root
    stack = []
    
    for line in lines:
        # line = line.split()
        line = line.strip()
        if not line:
            continue
        #以一个关键字开头，最后是一个冒号，可以有空格
        if re.match(r'^[a-zA-Z0-9 _]+\s*:', line):
            stack.append(current_node)
            new_node = {
                "name": line.strip(':'),
                "children": []
            }
            current_node["children"].append(new_node)
            current_node = new_node
        #以一个关键字开头，最后是一个冒号，可以有空格 ,冒号后面也是带数字的关键字
        elif re.match(r'^[a-zA-Z0-9_][a-zA-Z0-9_]*\s*:\s*[a-zA-Z0-9_][a-zA-Z0-9_]*\s*$', line):
            stack.append(current_node)
            ty = line.split(":")[0].strip()
            name = line.split(":")[1].strip()
            new_node = {
                "name": name,
                "type": ty,
                "children": []
            }
            current_node["children"].append(new_node)
            current_node = new_node
        #END
        elif re.match(r'^END$', line):
            if stack:
                current_node = stack[-1]
                stack.pop()
            else :
                current_node = root
        # 匹配任何"前面内容=后面内容"形式的赋值语句
        elif '=' in line:
            parts = line.split('=', 1)
            name = parts[0].strip()
            value = parts[1].strip()
            
            # 判断是否为数组带单位格式
            # array_with_unit_match = re.match(r'^(\[.*\])\s*(\[.*\])$', value)
            if re.match(r'^([a-zA-Z0-9+\-*. ]*)\[([^\[\]]+)\]$', value):
                # match = re.match(r'^[a-zA-Z0-9+\-*]*\[[a-zA-Z0-9+\-*]+\]$', value)
                # print("1",value)
                match = re.match(r'^([a-zA-Z0-9+\-*. ]*)\[([^\[\]]+)\]$', value)
                if match:
                    value_ = match.group(1) if match.group(1) else "1"
                
                unit = match.group(2)
                value_ = value_.strip()
                if len(value_) == 0:
                    value_ = "1"
                type = "str"
                if value_.lower() == "true":
                    value_ = True
                    type = "bool"
                elif value_.lower() == "false":
                    value_ = False
                    type = "bool"
                elif re.match(r'^\d+$', value_):
                    value_ = int(value_)
                    type = "int"
                elif re.match(r'^\d+\.\d*$', value_):
                    value_ = float(value_)
                    type = "real"
                #1.0e-5
                elif re.match(r'^\d+\.?\d*[eE][+-]?\d+$', value_):
                    value_ = float(value_)
                    type = "real"
                value_node = {
                    "name": name,
                    "value": value_,
                    "exprn": unit,
                    "type": type
                }
            elif re.match(r'^[+-]?(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?$', value):
                value_ = value.strip()
                type = "str"
                if value_.lower() == "true":
                    value_ = True
                    type = "bool"
                elif value_.lower() == "false":
                    value_ = False
                    type = "bool"
                elif re.match(r'^\d+$', value_):
                    value_ = int(value_)
                    type = "int"
                elif re.match(r'^\d+\.\d+$', value_):
                    value_ = float(value_)
                    type = "real"
                #1.0e-5
                elif re.match(r'^\d+\.?\d*[eE][+-]?\d+$', value_):
                    value_ = float(value_)
                    type = "real"
                # 处理负数浮点数
                elif re.match(r'^[+-]?(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?$', value_):
                    value_ = float(value_)
                    type = "real"
                value_node = {
                    "name": name,
                    "value": value_,
                    "type": type
                }
            elif re.match(r'^[^,\[\]]+(?:\s*,\s*[^,\[\]]+)+$', value):
                type = "str list"
                if re.match(r'^\d+(,\d+)*$', value):
                    value = [int(val) for val in value.split(",")]
                    type = "int list"
                elif re.match(r'^[+-]?\d+(?:\.\d+)?(?:[eE][+-]?\d+)?(?:\s*,\s*[+-]?\d+(?:\.\d+)?(?:[eE][+-]?\d+)?)+$', value):
                    value = [float(v) for v in value.split(",")]
                    type = "real list"
                else:
                    value = [v for v in value.split(",")]
                value_node = {
                    "name": name,
                    "value": value,
                    "type": type
                }
            elif re.match(r'(?:[^,\[\]]*?\[[^\[\]]*?\])(?:,(?:[^,\[\]]*?\[[^\[\]]*?\]))*', value):
                value_list = value.split(",")
                value_after_tt = []
                unit_list = []
                type = 0
                # print(value_list)
                for value in value_list:
                    # print(value)
                    match = re.match(r'^([^\[\]]+)\[([^\[\]]+)\]$', value)
                    value_ = match.group(1).strip()
                    unit = match.group(2).strip()
                    # print(value_)
                    if re.match(r'^\d+$', value_):
                        value_after_tt.append (int(value_))
                        type = max(type,1)
                    elif re.match(r'^\d+\.\d*$', value_):
                        value_after_tt.append (float(value_))
                        type = max(type,2)
                    #1.0e-5
                    elif re.match(r'^\d+\.?\d*[eE][+-]?\d+$', value_):
                        value_after_tt.append (float(value_))
                        type = max(type,2)
                    elif len(value_.strip()) == 0:
                        value_after_tt.append (1)
                        type = max(type,1)
                    unit_list.append(unit)
                if type > 1:
                    type = "real list"
                elif type >0 :
                    type = "int list"
                else :
                    type = "str list"
                value_node = {
                    "name": name,
                    "value": value_after_tt,
                    "exprn": unit_list,
                    "type": type
                }
                # print(value_node)
            else:
                value_node = {
                    "name": name,
                    "value": value,
                    "type": "str"
                }
            current_node["children"].append(value_node)
        else:
            print(f"无法解析的行: ss{line}ss")
            return None
    
    return root

def build_tree_from_ccl(path: str):
    with open(path, "r", encoding="utf-8") as f:
        config_text = f.read()
    root =  parse_ccl_to_dict(config_text)
    json_str = json.dumps(root, indent=4, ensure_ascii=False)
    # print(json_str)
    data = json.loads(json_str, object_pairs_hook=OrderedDict)
    with open("output2.txt", "w", encoding="utf-8") as f:
        f.write(json_str)
    manager = CclManager()
    tree = manager.trees["source"]
    root = tree.ROOT()
    for child_data in data.get('children', []):
        build_tree(root,child_data)
    return manager         
            


import json
import sqlite3


class MyDatabase:
    def __init__(self):
        self.conn = sqlite3.connect(':memory:')
        self.cursor = self.conn.cursor()
        self.init_relation_table()

    def init_relation_table(self):
        self.cursor.execute('''
        CREATE TABLE IF NOT EXISTS relations (
            tree_name TEXT NOT NULL,
            parent_id INTEGER NOT NULL,
            child_id INTEGER NOT NULL,
            relation_type TEXT,
            alias TEXT  
        )
        ''')
        self.conn.commit()

    def init_node_table(self, name):
        if not name.isidentifier():
            raise ValueError(f"非法的表名: {name}")

        sql = f'''
        CREATE TABLE IF NOT EXISTS {name} (
            id INTEGER PRIMARY KEY,
            node_name TEXT NOT NULL,
            node_type TEXT,
            node_value TEXT,
            node_exprn TEXT
        )
        '''
        self.cursor.execute(sql)
        self.conn.commit()

    def get_all_node_tables(self):
        query = '''
        SELECT name FROM sqlite_master
        WHERE type='table' AND name NOT IN ('users', 'relations')
        '''
        self.cursor.execute(query)
        result = [row[0] for row in self.cursor.fetchall()]
        return result

    def get_node(self, table_name, node_id):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")
        
        select_sql = f'''
        SELECT id, node_name, node_type, node_value, node_exprn FROM {table_name} WHERE id = ?
        '''
        self.cursor.execute(select_sql, (node_id,))
        result = self.cursor.fetchone()

        if result:
            id_, name, ty, value,node_exprn = result

            if ty == 'real list' and isinstance(value, str):
                value = json.loads(value)
            elif ty == 'int list' and isinstance(value, str):
                value = json.loads(value)
            return id_, name, ty, value, node_exprn
        return None
    
    def add_node(self, table_name, id, name, ty=None, value=None):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")
        
        if isinstance(value, (list, dict)):
            value = json.dumps(value)

        insert_sql = f'''
        INSERT INTO {table_name} (id, node_name, node_type, node_value)
        VALUES (?, ?, ?, ?)
        '''
        self.cursor.execute(insert_sql, (id, name, ty, value))
        self.conn.commit()
    
    def add_exprn(self,table_name, id, exprn):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")
        
        #exprn为字符串数组
        if isinstance(exprn, (list, dict)):
            exprn = json.dumps(exprn)

        #为已经存在的点添加exprn
        update_sql = f'''
        UPDATE {table_name}
        SET node_exprn = ?
        WHERE id = ?
        '''
        self.cursor.execute(update_sql, (id, exprn))
        self.conn.commit()
    


    def add_relation(self, tree_name, parent_id, child_id):
        insert_sql = '''
        INSERT INTO relations (tree_name, parent_id, child_id, relation_type, alias)
        VALUES (?, ?, ?, 'normal',?)
        '''
        self.cursor.execute(insert_sql, (tree_name, parent_id, child_id, None))
        self.conn.commit()
    
    def add_link(self, tree_name, parent_id, child_id, alias = None):
        if alias is None:
            select_name_sql = f'''
            SELECT node_name FROM {tree_name} WHERE id = ?
            '''
            self.cursor.execute(select_name_sql, (child_id,))
            row = self.cursor.fetchone()
            if row is None:
                raise ValueError(f"未找到 id={child_id} 的节点")
            alias = row[0]

        insert_sql = '''
        INSERT INTO relations (tree_name, parent_id, child_id, relation_type, alias)
        VALUES (?, ?, ?, 'link',?)
        '''
        self.cursor.execute(insert_sql, (tree_name, parent_id, child_id, alias))
        self.conn.commit()

    def find_child_by_name(self, tree_name, table_name, parent_id, target_name):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")

        # 先查 relations 表，找到 parent_id 的所有子节点 id
        select_children_sql = '''
        SELECT child_id FROM relations WHERE tree_name = ? AND parent_id = ?
        '''
        self.cursor.execute(select_children_sql, (tree_name, parent_id))

        child_ids = [row[0] for row in self.cursor.fetchall()]

        if not child_ids:
            return None  # 没有子节点

        # 再查 node 表（原生节点） + link（通过别名匹配）
        placeholder = ','.join(['?'] * len(child_ids))
        select_nodes_sql = f'''
        SELECT id, node_name, node_type, node_value, node_exprn
        FROM {table_name}
        WHERE id IN ({placeholder}) AND node_name = ?
        UNION
        SELECT r.child_id AS id, n.node_name, n.node_type, n.node_value , n.node_exprn
        FROM relations r
        JOIN {table_name} n ON r.child_id = n.id
        WHERE r.parent_id = ? AND r.tree_name = ? AND r.alias = ?
        '''

        self.cursor.execute(
            select_nodes_sql,
            (*child_ids, target_name, parent_id, tree_name, target_name)
        )

        result = self.cursor.fetchone()

        return result  # 可能是 tuple 或 None
    
    def get_children(self, tree_name, table_name, parent_id):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")

        select_children_sql = '''
        SELECT r.child_id, r.alias, n.node_name
        FROM relations r
        JOIN {table_name} n ON r.child_id = n.id
        WHERE r.tree_name = ? AND r.parent_id = ?
        '''.format(table_name=table_name)

        self.cursor.execute(select_children_sql, (tree_name, parent_id))
        rows = self.cursor.fetchall()

        if not rows:
            return None  # 没有子节点

        # 处理：如果 alias 存在就用 alias，否则用 node_name
        result = []
        for child_id, alias, node_name in rows:
            display_name = alias if alias else node_name
            result.append((child_id, display_name))

        return result


    def delete_node_and_children(self, table_name, node_id):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")

        select_sql = '''
        SELECT child_id FROM relations WHERE tree_name = ? AND parent_id = ?
        '''
        self.cursor.execute(select_sql, (table_name, node_id))
        children = [row[0] for row in self.cursor.fetchall()]

        for child_id in children:
            self.delete_node_and_children(table_name, child_id)

        delete_relations_sql = '''
        DELETE FROM relations WHERE tree_name = ? AND (parent_id = ? OR child_id = ?)
        '''
        self.cursor.execute(delete_relations_sql, (table_name, node_id, node_id))

        delete_node_sql = f'''
        DELETE FROM {table_name} WHERE id = ?
        '''
        self.cursor.execute(delete_node_sql, (node_id,))

        self.conn.commit()

    def get_node_value(self, table_name, node_id):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")

        select_sql = f'''
        SELECT node_value FROM {table_name} WHERE id = ?
        '''
        self.cursor.execute(select_sql, (node_id,))
        result = self.cursor.fetchone()

        if result:
            return result[0]
        return None
    
    def get_node_type(self, table_name, node_id):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")

        select_sql = f'''
        SELECT node_type FROM {table_name} WHERE id = ?
        '''
        self.cursor.execute(select_sql, (node_id,))
        result = self.cursor.fetchone()

        if result:
            return result[0]
        return None

    def get_node_exprn(self, table_name, node_id):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")

        select_sql = f'''
        SELECT node_exprn FROM {table_name} WHERE id = ?
        '''
        self.cursor.execute(select_sql, (node_id,))
        result = self.cursor.fetchone()

        if result:
            return result[0]
        return None

    def close(self):
        self.conn.close()
    
    def change_node(self, table_name, node_id, name=None, type=None, value=None):
        if not table_name.isidentifier():
            raise ValueError(f"非法的表名: {table_name}")

        fields = []
        values = []

        if name is not None:
            fields.append("node_name = ?")
            values.append(name)
        if type is not None:
            fields.append("node_type = ?")
            values.append(type)
        if value is not None:
            fields.append("node_value = ?")
            values.append(value)

        if not fields:
            # 没有字段要更新
            return True

        update_sql = f'''
        UPDATE {table_name}
        SET {', '.join(fields)}
        WHERE id = ?
        '''
        values.append(node_id)
        self.cursor.execute(update_sql, values)
        self.conn.commit()

        return True



import fnmatch
from .ccl_tt_node import CclTtNode,CclTtNodes
from .data_base import MyDatabase

class CclTtTree:
    def __init__(self, name: str, db: MyDatabase):
        self.name = name
        self.db = db
        self.nodes = {}
        self.node_id_counter = 1

        # 创建表
        self.db.init_node_table(name)

        # 创建 root
        root = CclTtNode(self, node_id=self.node_id_counter, name = "root")
        self.nodes[self.node_id_counter] = root
        self.root = root

        self.db.add_node(self.name, root.node_id, root.name)

    def ROOT(self):
        return CclTtNodes([self.root])

    def build_tree(self, parent_id, name,ty = None, value = None):
        self.node_id_counter += 1
        node = CclTtNode(self, node_id=self.node_id_counter,name = name)
        self.nodes[self.node_id_counter] = node

        self.db.add_node(self.name, node.node_id, node.name, ty, value)
        self.db.add_relation(self.name, parent_id, node.node_id)

        return node.node_id

    def add_path(self, parent_id: int, path: str) -> 'CclTtNode':
        now =CclTtNode(self, node_id=parent_id)
        if now.type() is not None and now.value() is not None:
            return False
        # Split the path by '/' to get the directory names
        path_parts = path.split('/')
        
        current_parent_id = parent_id

        error = False
        # Iterate over each part of the path
        for part in path_parts:
            # Check if the node for this part already exists
            if error:
                return False
            node = self.db.find_child_by_name(self.name,self.name,current_parent_id, part)
            if node is None:  # If node does not exist, create it
                node = self.build_tree(current_parent_id, part)
                current_parent_id = node
            else:
                node_id, node_name, node_type, node_value, node_exprn = node
                if node_type is None and node_value is None:
                    current_parent_id = node_id
                else:
                    error = True
            # Set the parent to the newly created or found node
            
        if error:
            return False
        return True
    
    def add_val(self, parent_id: int, path: str, ty, val) -> 'CclTtNode':
        now =CclTtNode(self, node_id=parent_id)
        if now.type() is not None and now.value() is not None:
            return False
        # Split the path by '/' to get the directory names
        path_parts = path.split('/')
        
        current_parent_id = parent_id
        i = len(path_parts)
        error = False
        # Iterate over each part of the path
        for part in path_parts:
            i = i - 1
            # Check if the node for this part already exists
            if error:
                return False
            node = self.db.find_child_by_name(self.name,self.name,current_parent_id, part)
            if i == 0:
                if node is None:
                    node = self.build_tree(current_parent_id, part, ty, val)
                    return True
                else:
                    return False
            elif node is None:  # If node does not exist, create it
                node = self.build_tree(current_parent_id, part)
                current_parent_id = node
            else:
                node_id, node_name, node_type, node_value = node
                if node_type is None and node_value is None:
                    current_parent_id = node_id
                else:
                    error = True
            # Set the parent to the newly created or found node
            
        if error:
            return False
        return True
    
    def add_link(self, parent_id: int, child: int, alias: str | None) -> 'CclTtNode':
        now =CclTtNode(self, node_id=parent_id)
        if now.type() is not None and now.value() is not None:
            return False
        # Split the path by '/' to get the directory names
        node = self.db.find_child_by_name(self.name,self.name,parent_id, alias)
        if not node is None:
            return False
        self.db.add_link(self.name, parent_id, child, alias)
        return True

    def find_by_path_with_wildcard(self, start_node_id, path: str):
        path_parts = path.split('/')  # 例如 ["a*", "b"]
        current_nodes = [(start_node_id, "no")]  # 初始是根节点

        for part in path_parts:
            next_nodes = []
            for node in current_nodes:
                # 找到 node 的所有子节点
                node_id,node_name = node
                children = self.db.get_children(self.name,self.name,node_id)
                # 根据通配符匹配
                if not children is None:
                    matched = [child for child in children if fnmatch.fnmatch(child[1], part)]

                    next_nodes.extend(matched)

            current_nodes = next_nodes

            if not current_nodes:
                break  # 中途没找到，提前结束
        
        ans = [self.nodes[node[0]] for node in current_nodes]
        return ans

    def deep_copy(self, parent_id: int, node_id: int, source_tree = None):
        # 复制节点
        tree_name = self.name if source_tree is None else source_tree
        parent = self.db.get_node(self.name,parent_id)
        node = self.db.get_node(tree_name,node_id)
        if node is None:
            return False
        if parent[2] is not None and parent[3] is not None:
            return False
        if  self.find_by_path_with_wildcard(parent_id, node[1]):
            return False
        # print(node[1],node[2],node[3])
        new_node_id = self.build_tree(parent_id, node[1], node[2], node[3])
        # 复制子节点
        children = self.db.get_children(tree_name,tree_name,node_id)
        if not children is None:
            for child in children:
                if(not self.deep_copy(new_node_id, child[0],source_tree)):
                    return False
        return True
    
    def delete_node_and_children(self, node_id):
        self.db.delete_node_and_children(self.name, node_id)
        del self.nodes[node_id]

    def add_exprn(self,node_id,exprn):
        self.db.add_exprn(self.name, node_id, exprn)

    def rename(self,node_id,name):
        self.db.change_node(self.name,node_id=node_id,name=name,type=None,value=None)
    
    def retype(self,node_id,type):
        self.db.change_node(self.name,node_id=node_id,name=None,type=type,value=None)
    
    def revalue(self,node_id,value):
        self.db.change_node(self.name,node_id=node_id,name=None,type=None,value=value)
    



import copy

def format_values(exprn, value):
    # 单值情况
    print("exprn",exprn)
    print("value",value)
    if not isinstance(exprn, list):
        return f"{value}[{exprn}]"
    
    # 列表情况
    if isinstance(exprn, list) and isinstance(value, list):
        if len(exprn) != len(value):
            raise ValueError("exprn 和 value 长度不一致")
        return [f"{v}[{e}]" for v, e in zip(value, exprn)]
    
    raise TypeError("exprn 和 value 类型不匹配")

class CclTtNode:
    def __init__(self, tree, node_id: int, name: str = None):
        self.tree = tree
        self.node_id = node_id
        self.name = name

    def __str__(self):
        return f"CclTtNode (id={self.node_id} name={self.name})"
    
    def value(self):
        return self.tree.db.get_node_value(self.tree.name, self.node_id)
    
    def type(self):
        return self.tree.db.get_node_type(self.tree.name, self.node_id)
    
    def exprn(self):
        return self.tree.db.get_node_exprn(self.tree.name, self.node_id)

    def __repr__(self):
        return self.__str__()
    
    def add(self, path: str):
        """自己添加一个子节点"""
        return self.tree.add_path(self.node_id, path)
    def get(self, path: str):
        return self.tree.find_by_path_with_wildcard(self.node_id, path)
    
    def retype(self, type: str):
        self.tree.retype(type)
    
    def revalue(self, value):
        self.tree.revalue(value)

    def rename(self, name):
        self.name = name
        self.tree.rename(name)
    
    def print(self,indent):
        ans = indent
        ans += self.name
        cnt = 0
        if self.type():
            ans += " : "
            ans += self.type()
            cnt = 1
        if self.value():
            ans += " = "
            exprn = self.exprn()
            value = "\"" + self.value() + "\"" if self.type() == "str" else self.value()
            if exprn is not None:
                ans += format_values(exprn, value)
            else:
                ans += str(value)
            cnt = 1
        if cnt == 1 :
            return ans
        children = self.tree.find_by_path_with_wildcard(self.node_id,"*")
        if children:
            ans += '\n'
            ans += indent
            ans += "children :"
            for child in children:
                ans += '\n'
                ans += child.print(indent + "    ")
        return ans    
    def transfrom(self,indent):
        ans = indent
        ans += self.name
        if self.value() is not None and self.type() is not None :
            ans += " = "
            value = "\"" + self.value() + "\"" if self.type() == "str" else self.value()
            exprn = self.exprn()
            if exprn is not None:
                ans += format_values(exprn, value)
            else:
                ans += str(value)
            return ans
        ans += "\n"
        ans += indent
        ans += "{"
        for child in self.tree.find_by_path_with_wildcard(self.node_id,"*"):
            ans += '\n'
            ans += child.transfrom(indent + "    ")
        ans += '\n'
        ans += indent
        ans += "}"
        return ans

class CclTtNodes:
    def __init__(self, nodes):
        self.nodes= nodes
    

    def __str__(self):
        return f"CclTtNodes (nodes: {self.nodes})"

    def __repr__(self):
        return self.__str__()

    def value(self):
        if len(self.nodes) != 1:
            return None
        return self.nodes[0].value()
    
    def type(self):
        if len(self.nodes) != 1:
            return None
        return self.nodes[0].type()

    def add(self, path, val = None):
        """自己添加一个子节点"""
        if not isinstance(path, str) and not isinstance(path, CclTtNodes):
            if self.add_link(val, path):
                return True
            return False
        elif val is not None:
            if self.add_val(path, val):
                return True
            return False
        elif isinstance(path, str):
            if self.add_dir(path):
                return True
            return False
        elif isinstance(path, CclTtNodes):
            if self.add_tree(path):
                return True
            return False
        
    
    def add_dir(self, path: str):
        """自己添加一个子节点"""
        for node in self.nodes:
            if not node.tree.add_path(node.node_id, path):
                return False
        return True
    
    def add_tree(self, nodes):
        if len(self.nodes) != 1:
            return False
        root_node = self.nodes[0]
        for node in nodes.nodes:
            if node.tree.name != root_node.tree.name:
                if( not root_node.tree.deep_copy(root_node.node_id, node.node_id, source_tree = node.tree.name)):
                    return False
            else:
                if( not root_node.tree.deep_copy(root_node.node_id, node.node_id)):
                    return False
        return True
    
    def add_val(self, path ,val):
        if isinstance(val, bool):
            for node in self.nodes:
                if not node.tree.add_val(node.node_id, path,"bool",val):
                    return False
            return True
        elif isinstance(val, str):
            for node in self.nodes:
                if not node.tree.add_val(node.node_id, path,"str",val):
                    return False
            return True
        elif isinstance(val, int):
            for node in self.nodes:
                if not node.tree.add_val(node.node_id, path,"int",val):
                    return False
            return True
        elif isinstance(val, float):
            for node in self.nodes:
                if not node.tree.add_val(node.node_id, path,"real",val):
                    return False
            return True
        elif isinstance(val, list) and all(isinstance(x, int) for x in val):
            for node in self.nodes:
                if not node.tree.add_val(node.node_id, path,"int list",val):
                    return False
            return True
        elif isinstance(val, list) and all(isinstance(x, float) for x in val):
            for node in self.nodes:
                if not node.tree.add_val(node.node_id, path,"real list",val):
                    return False
            return True

    def add_link(self, node, alias: str | None):
        if len(self.nodes) != 1:
            return False
        if len(node.nodes) != 1:
            return False
        root_node = self.nodes[0]
        add_node = node.nodes[0]
        if( not root_node.tree.add_link(root_node.node_id, add_node.node_id, alias)):
            return False
        return True
    
    def get(self, path: str):
        ans = []
        for node in self.nodes:
            ans.extend(node.tree.find_by_path_with_wildcard(node.node_id, path))
        if len(ans) == 0:
            return None
        return CclTtNodes(ans)
    
    def name(self):
        if len(self.nodes) == 1:
            return self.nodes[0].name
        return None
    
    def copy(self):
        return CclTtNodes(copy.copy(self.nodes))
    
    def delete(self, path = None):
        """删除节点"""
        if path is  None:
            for node in self.nodes:
                node.tree.delete_node_and_children(node.node_id)
            del self
            return
        node = self.get(path)
        if node is None:
            return
        node.delete()
        return 
    
    def rename(self, name1: str ,name2 :str = None):
        if name2 is None:
            if len(self.nodes) == 1:
                self.nodes[0].name = name1
                self.nodes[0].tree.rename(self.nodes[0].node_id,name1)
                return True
            else:
                return False
        else:
            chul_node =  self.get(name1)
            if chul_node is None:
                return False
            chul_node.rename(name2)
            return True
        
    def retype(self, type: str):
        if len(self.nodes) == 1:
            self.nodes[0].tree.retype(self.nodes[0].node_id,type)
            return True
        else:
            return False
    
    def revalue(self, value):
        if len(self.nodes) == 1:
            self.nodes[0].tree.revalue(self.nodes[0].node_id,value)
            return True
        else:
            return False
        
    def cclnodes(self):
        res = []
        for i in self.nodes:
            res.append(CclTtNodes([i]))
        return res
    
    def print(self):
        ans = ""
        for node in self.nodes:
            ans += node.print("")
            ans += '\n\n'
        return ans

    def transfrom(self):
        ans = ""
        for node in self.nodes:
            ans += node.transfrom("")
            ans += "\n"
        return ans
    
    def add_exprn(self,exprn):
        if len(self.nodes) != 1:
            return False
        root_node = self.nodes[0]
        if( not root_node.tree.add_exprn(root_node.node_id,exprn)):
            return False
        return True

        

def merge(a:CclTtNodes, b:CclTtNodes, c = None):
    """两个节点的并集"""
    if a is None:
        return b
    if b is None:
        return a
    return CclTtNodes(a.nodes + b.nodes)

def remove(a:CclTtNodes, b:CclTtNodes):
    a_copy = a.copy()
    for node in a_copy.nodes:
        for node2 in b.nodes:
            if node.node_id == node2.node_id and node.tree.name == node2.tree.name:
                a_copy.nodes.remove(node)
    return a_copy


# SPDX-FileCopyrightText: 2025-present Flyshde <zhangyang@outlook.es>
#
# SPDX-License-Identifier: MIT
from .ccl_tt_tree import CclTtTree
from .ccl_tt_node import CclTtNodes,merge,remove
from .ccl_manager import CclManager, build_tree_from_json, build_tree_from_ccl

__all__ = ["CclTtTree", "CclTtNodes", "CclManager", "merge", "remove","build_tree_from_json","build_tree_from_ccl"]


from src.ccltt.ccl_manager import CclManager, build_tree_from_ccl, build_tree_from_json
from src.ccltt.ccl_tt_tree import CclTtTree
from src.ccltt.ccl_tt_node import merge,remove
# Removed redundant import as merge is already part of CclTtNodes

def test_initial_tree():
    print("")
    print("test_initial_tree")
    root = CclTtTree("source")
    assert root.num == 1
    assert root.root.index == 1 
    print("")

def test_initial_tree():
    print("")
    print("test_initial_tree")
    manager = CclManager()
    assert manager.trees["source"].name == "source"
    assert manager.db.get_all_node_tables()[0] == "source"
    print("")

def test_add_dir():
    manager = CclManager()
    tree = manager.trees["source"]
    root = tree.ROOT()
    assert root.name() == "root"
    print("")
    print("test_add_dir")
    assert root.add("a/w")
    assert root.add("a/d")
    assert root.add("a/df/ccc")
    assert root.add("a/ddg/ca")
    assert root.add("a/d/c")
    assert root.add("a/df/cb")
    a = merge(root.get("a/df/c*"),root.get("a/ddg/c*"))
    assert a.__str__() == "CclTtNodes (nodes: [CclTtNode (id=6 name=ccc), CclTtNode (id=10 name=cb), CclTtNode (id=8 name=ca)])"
    b = root.get("a/ddg/c*")
    c = remove(a,b)
    assert a.__str__() == "CclTtNodes (nodes: [CclTtNode (id=6 name=ccc), CclTtNode (id=10 name=cb), CclTtNode (id=8 name=ca)])"
    assert c.__str__() == "CclTtNodes (nodes: [CclTtNode (id=6 name=ccc), CclTtNode (id=10 name=cb)])"
    print("")

def test_add_tree():
    manager = CclManager()
    tree = manager.trees["source"]
    root = tree.ROOT()
    assert root.name() == "root"
    print("")
    print("test_add_tree")
    assert root.add("a/w")
    assert root.add("a/d")
    assert root.add("a/df/ccc")
    assert root.add("a/ddg/ca")
    assert root.add("a/d/c")
    assert root.add("a/df/cb")
    a = root.get("a/d")
    b = root.get("a/w") 
    b.add(a)
    assert b.get("*").__str__() == "CclTtNodes (nodes: [CclTtNode (id=11 name=d)])"
    assert b.get("*/*").__str__() == "CclTtNodes (nodes: [CclTtNode (id=12 name=c)])"
    assert a.get("*").__str__() == "CclTtNodes (nodes: [CclTtNode (id=9 name=c)])"
    assert root.get("a/*").__str__() == "CclTtNodes (nodes: [CclTtNode (id=3 name=w), CclTtNode (id=4 name=d), CclTtNode (id=5 name=df), CclTtNode (id=7 name=ddg)])"
    print("")

def test_delete_tree():
    manager = CclManager()
    tree = manager.trees["source"]
    root = tree.ROOT()
    assert root.name() == "root"
    print("")
    print("test_delete_tree")
    assert root.add("a/w")
    assert root.add("a/d")
    assert root.add("a/df/ccc")
    assert root.add("a/ddg/ca")
    assert root.add("a/d/c")
    assert root.add("a/df/cb")
    a = root.get("a/d")
    b = root.get("a/w") 
    c = merge(a,b)
    c.delete("c")
    assert root.get("a/*/*").__str__() == "CclTtNodes (nodes: [CclTtNode (id=6 name=ccc), CclTtNode (id=10 name=cb), CclTtNode (id=8 name=ca)])"
    print("")

def test_rename_node():
    manager = CclManager()
    tree = manager.trees["source"]
    root = tree.ROOT()
    assert root.name() == "root"
    print("")
    print("test_rename_node")
    assert root.add("a/w")
    assert root.add("a/d")
    assert root.add("a/df/ccc")
    assert root.add("a/ddg/ca")
    assert root.add("a/d/c")
    assert root.add("a/df/cb")
    a = root.get("a/d")
    b = root.get("a/w") 
    # a.rename("d1")
    c = merge(a,b)
    assert c.rename("d1") == False
    assert root.get("a/*").__str__() == "CclTtNodes (nodes: [CclTtNode (id=3 name=w), CclTtNode (id=4 name=d), CclTtNode (id=5 name=df), CclTtNode (id=7 name=ddg)])"
    # print(root.get("*/*"))
    assert a.get("*").__str__() == "CclTtNodes (nodes: [CclTtNode (id=9 name=c)])"
    print("")

def test_add_val():
    manager = CclManager()
    tree = manager.trees["source"]
    root = tree.ROOT()
    assert root.name() == "root"
    print("")
    print("test_add_val")
    assert root.add("a/w")
    assert root.add("a/d")
    assert root.add("a/df/ccc")
    assert root.add("a/ddg/ca")
    assert root.add("a/d/c")
    assert root.add("a/df/cb")
    a = root.get("a/d")
    b = root.get("a/w") 
    c = merge(a,b)
    assert c.__str__() == "CclTtNodes (nodes: [CclTtNode (id=4 name=d), CclTtNode (id=3 name=w)])"
    assert a.add_val("d1",True)
    assert a.add_val("d2","1")
    assert a.add_val("d3",0)
    assert a.add_val("d4",1.0)
    assert a.get("d1").value()
    assert a.get("d1").type() == "bool"
    assert a.get("d2").value() == "1"
    assert a.get("d2").type() == "str"
    assert a.get("d3").value() == "0"
    assert a.get("d3").type() == "int"
    assert a.get("d4").value() == "1.0"
    assert a.get("d4").type() == "real"
    assert a.get("d1").value() == "1"
    assert a.get("d1").add("x") == False
    e = a.get("c")
    e.add("wwww/eee/w",1)

    tree2 = manager.init_tree("target")
    root2 = tree2.ROOT()
    assert root2.name() == "root"
    assert root2.add(a)
    print(root2.print())
    # assert root2.get("d").__str__() == "CclTtNodes (nodes: [CclTtNode (id=2 name=d)])"
    # assert root2.get("d").get("*").__str__() == "CclTtNodes (nodes: [CclTtNode (id=3 name=c), CclTtNode (id=4 name=d1), CclTtNode (id=5 name=d2), CclTtNode (id=6 name=d3), CclTtNode (id=7 name=d4)])"
    print("")

def test_add_link():
    manager = CclManager()
    tree = manager.trees["source"]
    root = tree.ROOT()
    assert root.name() == "root"
    print("")
    print("test_add_link")
    assert root.add("a/w")
    assert root.add("a/d")
    assert root.add("a/df/ccc")
    assert root.add("a/ddg/ca")
    assert root.add("a/d/c")
    assert root.add("a/df/cb")
    a = root.get("a/d")
    b = root.get("a/w") 
    # c = merge(a,b)
    assert b.add_link(a,"link") == True
    assert b.get("link").name() == "d"
    assert b.get("link").rename("link1")
    assert b.get("link").name() == "link1"

def test_build_from_json():
    print("")
    print("test_build_from_json")
    manager = build_tree_from_json("aesim_cb.input")
    tree = manager.trees["source"]
    root = tree.ROOT()
    with open("json_output.txt", "w", encoding="utf-8") as f:
        f.write(root.print())
    with open("json_transfrom.txt", "w", encoding="utf-8") as f:
        f.write(root.get("*").transfrom())
    # print(root.print())
    # print(root.get("*").transfrom())
    # print(root.get("periodicity_1/rotation_invariant_point").type())
    # print(root.get("*/*"))
    # print(root.get("*/*/*"))
    # print(root.get("*/*/*/*"))

def test_build_from_ccl_8_23():
    print("")
    print("test_build_from_ccl_8_23")
    manager = build_tree_from_json("8-23.input")
    tree = manager.trees["source"]
    root = tree.ROOT()
    with open("8-23_json_output.txt", "w", encoding="utf-8") as f:
        f.write(root.print())
    with open("8-23_json_transfrom.txt", "w", encoding="utf-8") as f:
        f.write(root.get("*").transfrom())
    # print(root.print())
    # print(root.get("*").transfrom())
    # print(root.get("periodicity_1/rotation_invariant_point").type())
    # print(root.get("*/*"))
    # print(root.get("*/*/*"))
    # print(root.get("*/*/*/*"))

def test_build_from_ccl():
    print("")
    print("test_build_from_json")
    manager = build_tree_from_ccl("ccl")
    # print(manager)
    tree = manager.trees["source"]
    root = tree.ROOT()
    with open("ccl_output.txt", "w", encoding="utf-8") as f:
        f.write(root.print())

def test_build_from_ccl_8_22():
    print("")
    print("test_build_from_ccl_8_22")
    manager = build_tree_from_ccl("8-23")
    # print(manager)
    tree = manager.trees["source"]
    root = tree.ROOT()
    with open("8-23_output.txt", "w", encoding="utf-8") as f:
        f.write(root.print())

def test_retype_node():
    manager = CclManager()
    tree = manager.trees["source"]
    root = tree.ROOT()
    assert root.name() == "root"
    print("")
    print("test_add_val")
    assert root.add("a/w")
    assert root.add("a/d")
    assert root.add("a/df/ccc")
    assert root.add("a/ddg/ca")
    assert root.add("a/d/c")
    assert root.add("a/df/cb")
    a = root.get("a/d")
    b = root.get("a/w") 
    c = merge(a,b)
    assert c.__str__() == "CclTtNodes (nodes: [CclTtNode (id=4 name=d), CclTtNode (id=3 name=w)])"
    assert a.add_val("d1",True)
    assert a.add_val("d2","1")
    assert a.add_val("d3",0)
    assert a.add_val("d4",1.0)
    assert a.get("d1").value()
    assert a.get("d1").type() == "bool"
    assert a.get("d2").value() == "1"
    assert a.get("d2").type() == "str"
    assert a.get("d3").value() == "0"
    assert a.get("d3").type() == "int"
    assert a.get("d4").value() == "1.0"
    assert a.get("d4").type() == "real"
    assert a.get("d1").value() == "1"
    assert a.get("d1").add("x") == False
    e = a.get("d3")
    assert e.type() == "int"
    assert e.value() == '0'
    e.retype("real")
    assert e.type() == "real"
    e.revalue(1)
    assert e.value() == '1'
    assert e.name() == "d3"
    w = a.get("*")
    for i in w.cclnodes():
        print(i.name())
    # assert root2.get("d").__str__() == "CclTtNodes (nodes: [CclTtNode (id=2 name=d)])"
    # assert root2.get("d").get("*").__str__() == "CclTtNodes (nodes: [CclTtNode (id=3 name=c), CclTtNode (id=4 name=d1), CclTtNode (id=5 name=d2), CclTtNode (id=6 name=d3), CclTtNode (id=7 name=d4)])"
    print("")
